#define BUILD_NUMBER \
	"build-19345655"
#define BUILD_NUMBER_NUMERIC \
	19345655
#define BUILD_NUMBER_NUMERIC_STRING \
	"19345655"
#define PRODUCT_BUILD_NUMBER \
	"product-build-34242"
#define PRODUCT_BUILD_NUMBER_NUMERIC \
	34242
#define PRODUCT_BUILD_NUMBER_NUMERIC_STRING \
	"34242"
